from io import BytesIO
from zipfile import ZipFile
from urllib.request import urlopen, Request
from bs4 import BeautifulSoup

def getTestData(problem):
    """
    Returns a dictionary on the form {input: correct output} from Kattis.
    """
    url = f"https://open.kattis.com/problems/{problem}/file/statement/samples.zip"
    request = Request(url, headers={'User-Agent': 'Chrome Browser'})
    resp = urlopen(request)
    zipfile = ZipFile(BytesIO(resp.read()))
    data = ["".join([line.decode("UTF-8") for line in zipfile.open(name).readlines()]) for name in zipfile.namelist()]
    testdata = {}
    for i in range(0, len(data), 2):
        if i < len(data) - 1:
            testdata[data[i]] = data[i + 1]
    return testdata

def CF(problem):
    """
    Returns a dictionary on the form {input: correct output} from Code Forces
    """
    num, let = problem[:-1], problem[-1]
    url = f"https://codeforces.com/problemset/problem/{num}/{let}"
    request = Request(url, headers={'User-Agent': 'Chrome Browser'})
    resp = urlopen(request)
    soup = BeautifulSoup(resp, 'html.parser')
    io = soup.find("div", class_="sample-test").text.split('\n')
    testdata = {}
    i = 0
    while i != len(io) - 1:
        if io[i] == "Input":
            input = ""
            i += 1
            while io[i] != "Output":
                input += io[i] + '\n'
                i += 1
            output = ""
            i += 1
            while io[i] != "Input":
                if i == len(io) - 1:
                    break
                output += io[i] + '\n'
                i += 1
            testdata[input] = output
    return testdata


from kattest.languages import python, cpp, c, java
from kattest.data import getTestData, CF
import emoji
import sys

def kattest(filename, kattis):
    """
    Method for running your code and returning output on the form {input: code output}
    Comparing user output with correct output
    """
    problem, extension = filename.split(".")
    if kattis:
        testdata = getTestData(problem.lower())
    else:
        testdata = CF(problem)
    if extension == "py":
        output, time = python(filename, testdata)
    elif extension == "cpp":
        output, time = cpp(filename, testdata)
    elif extension == "c":
        output, time = c(filename, testdata)
    elif extension == "java":
        output, time = java(filename, testdata)
    else:
        return "Language is not supported :("
    counter, correct_count = 1, 0
    for out in output:
        correct = testdata[out].split("\n")
        user = output[out].split("\n")
        if formatter(user) == formatter(correct):
            print(f'{emoji.emojize(":white_check_mark:", use_aliases=True)} Sample Input {counter}')
            correct_count += 1
        else:
            print(f'{emoji.emojize(":x:", use_aliases=True)} Sample Input {counter}')
        print("INPUT")
        print(out)
        print("CORRECT OUTPUT")
        print(testdata[out])
        print("USER OUTPUT")
        print(output[out])
        print("------------------------------")
        counter += 1
    print(f"Time: {time} seconds")
    print(f"Verdict: {correct_count}/{counter-1} correct cases")

def formatter(output):
    return [out.rstrip() for out in output if out != '']


import sys
from kattest.script import kattest

def main():
    try:
        cmd = sys.argv[1:]
        if cmd[0] == 'CF':
            kattest(cmd[1], False)
        else:
            kattest(cmd[0], True)
    except:
        print("Could not run your code.")
        
if __name__ == "__main__":
    main()


#!/usr/bin/env python
# encoding: UTF-8

import os
from setuptools import setup, find_packages
from setuptools.command.install import install as _install


class install(_install):
    def run(self):
        _install.run(self)

long_description = ""

try:
    if os.path.isfile("README.rst"):
        long_description = open("README.rst", "r").read()
    elif os.path.isfile("README.md"):
        long_description = open("README.md", "r").read()
except Exception as error:
    print("Unable to read the README file: " + str(error))

setup(
    name="kattest",
    version="0.6",
    description="Script for testing your code with the sample data files provided by Kattis.",
    url="https://github.com/dilawarm/kattest",
    license="MIT",
    long_description=long_description,
    author_email="dilawarmm@outlook.com",
    author="Dilawar Mahmood",
    packages=find_packages(),
    download_url = "https://github.com/dilawarm/kattest/archive/v_02.tar.gz",
    install_requires=[
        'emoji',
    ],
    entry_points={
        "console_scripts": [
            "kattest = kattest.__main__:main"
        ]
    },
    cmdclass={"install": install}
)


from subprocess import check_output
from sys import executable
import time

def python(filename, inputs):
    output = {}
    startTime = time.time()
    for input in inputs:
        output[input] = check_output([executable, filename], input=input, universal_newlines=True)
    endTime = time.time() - startTime
    return output, endTime

def cpp(filename, inputs):
    output = {}
    problem = filename.split(".")[0]
    check_output(["g++", filename, "-g", "-O2", "-o", f"{problem}"])
    startTime = time.time()
    for input in inputs:
        output[input] = check_output([f"./{problem}"], input=input, universal_newlines=True)
    endTime = time.time() - startTime
    check_output(["rm", "-rf", f"{problem}"])
    return output, endTime

def c(filename, inputs):
    output = {}
    problem = filename.split(".")[0]
    check_output(["gcc", filename, "-g", "-O2", "-o", f"{problem}"])
    startTime = time.time()
    for input in inputs:
        output[input] = check_output([f"./{problem}"], input=input, universal_newlines=True)
    endTime = time.time() - startTime
    check_output(["rm", "-rf", f"{problem}"])
    return output, endTime

def java(filename, inputs):
    output = {}
    problem = filename.split(".")[0]
    check_output(["javac", "-encoding", "UTF-8", filename])
    startTime = time.time()
    for input in inputs:
        output[input] = check_output(["java", "-Dfile.encoding=UTF-8", "-XX:+UseSerialGC", "-Xss64m", f"{problem}"], 
                                      input=input, universal_newlines=True)
    endTime = time.time() - startTime
    check_output(["rm", "-rf", f"{problem}.class"])
    return output, endTime